from google.auth import impersonated_credentials as gcp_impersonated_credentials
from googleapiclient.discovery import build


class ImpersonatedServiceAccount:
    def __init__(self, impersonator_credentials, the_one_you_want_to_impersonate, target_scopes, service_name, version) -> None:
        self.impersonated_credentials = gcp_impersonated_credentials.Credentials(
            source_credentials=impersonator_credentials,
            target_principal=the_one_you_want_to_impersonate,
            target_scopes=target_scopes,
            lifetime=500
        )

        self.service = build(service_name, version,
                             credentials=self.impersonated_credentials)

    def __enter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        pass



from google.auth.credentials import Credentials
from googleapiclient.discovery import build
from gcp_impersonation_wrapper.impersonated_service_account import ImpersonatedServiceAccount


class ServiceAccount:
    def __init__(self, project_id, impersonator_credentials: Credentials) -> None:
        self.project_id = project_id
        self.impersonator_credentials = impersonator_credentials
        self.service = build(
            'iam', 'v1', credentials=self.impersonator_credentials)
        self.service_account_email = self.impersonator_credentials._service_account_email

    def impersonate(self, service_account_email: str, target_scopes: list, service: str, version: str) -> None:
        return ImpersonatedServiceAccount(self.impersonator_credentials, service_account_email, target_scopes, service, version)

    def list_all_impersonatable_service_account_emails(self) -> list:
        impersonatable_service_accounts = self.list_all_impersonatable_service_account_objects()
        emails = self.get_emails(impersonatable_service_accounts)
        emails = list(emails)
        return emails

    def list_all_impersonatable_service_account_objects(self) -> list:
        self.service_accounts = self.service.projects().serviceAccounts().list(
            name=f"projects/{self.project_id}").execute()
        impersonatable_service_accounts = self.filter_impersonatable_service_accounts(
            self.service_accounts)
        impersonatable_service_accounts = list(impersonatable_service_accounts)
        return impersonatable_service_accounts

    def is_impersonatable(self, service_account_email: str) -> bool:
        policies = self.service.projects().serviceAccounts().getIamPolicy(
            resource=f"projects/{self.project_id}/serviceAccounts/{service_account_email}"
        ).execute()
        if "bindings" not in policies:
            return False
        for binding in policies["bindings"]:
            if "role" in binding and binding["role"] == "roles/iam.serviceAccountTokenCreator" and f"serviceAccount:{self.service_account_email}" in binding["members"]:
                return True
        return False

    def filter_impersonatable_service_accounts(self, service_accounts: str) -> list:
        for service_account in service_accounts["accounts"]:
            if self.is_impersonatable(service_account["email"]):
                yield service_account

    def get_emails(self, service_accounts):
        for service_account in service_accounts:
            yield service_account["email"]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gcp-impersonation-wrapper",
    version="0.0.4",
    packages=setuptools.find_packages(),
    long_description_content_type="text/markdown",
    long_description=long_description,
    python_requires=">=3.7",
    install_requires=[
        "requests",
        "httplib2",
        "google-api-core",
        "google-api-python-client",
        "google-auth",
        "google-cloud-core",
        "google-resumable-media",
        "googleapis-common-protos",
        "google-auth-oauthlib",
        "oauth2client",
    ],
    url="https://gitlab.com/dqna/dv-data-generator",
)
